-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

--[[ map related functions ]]--

local tinsert, tremove, ceil, sin, cos, tonumber, UnitPosition = table.insert, table.remove, math.ceil, math.sin, math.cos, tonumber, UnitPosition

local addonName, addonTable = ...

local T = addonTable.T
local AddColor = addonTable.AddColor

local map_data = setmetatable({}, { __index = function(t, k)
											   local instanceID, _, _, left, right, top, bottom = GetAreaMapInfo(k)
										       if instanceID then
											    t[k] = { id = instanceID, l = left, t = top, r = right, b = bottom, w = right - left, h = bottom - top }
											    return t[k]
											   else
											    t[k] = false
											   end
											  end})

-- zone names cache, available to plugins
DailyGlobalCheck.Z = setmetatable({}, { __index = function(t, k)
												   local result = GetMapNameByID(k) or ""
											       t[k] = result
												   return result
 											      end})

local lists =  DailyGlobalCheck.ListFunctions
local maps  = DailyGlobalCheck.MapFunctions

local minimapShapes = {
	-- {upper-left, lower-left, upper-right, lower-right}
	-- true = rounded, false = squared
	--["ROUND"]                 = {true , true , true , true}, -- no need of this
	["SQUARE"] 			      = {false, false, false, false},
	["CORNER-TOPLEFT"] 		  = {true , false, false, false},
	["CORNER-TOPRIGHT"] 	  = {false, false, true , false},
	["CORNER-BOTTOMLEFT"] 	  = {false, true , false, false},
	["CORNER-BOTTOMRIGHT"]	  = {false, false, false, true},
	["SIDE-LEFT"] 			  = {true , true , false, false},
	["SIDE-RIGHT"] 			  = {false, false, true , true},
	["SIDE-TOP"] 			  = {true , false, true , false},
	["SIDE-BOTTOM"] 		  = {false, true , false, true},
	["TRICORNER-TOPLEFT"] 	  = {true , true , true , false},
	["TRICORNER-TOPRIGHT"]    = {true , false, true , true},
	["TRICORNER-BOTTOMLEFT"]  = {true , true , false, true},
	["TRICORNER-BOTTOMRIGHT"] = {false, true , true , true},
}

local wmf = CreateFrame("Frame", "DGCWorldMapOverlay", WorldMapButton)
local mmf = CreateFrame("Frame", "DGCMinimapOverlay", Minimap)
local frames_pool = {}
local updates = {} -- option update functions
local icons = { worldmap = {}, minimap = {} }
local player_mapID, player_instanceID
local dungeon_data = {}
-- minimap vars
local player_last_x, player_last_y, lastZoom, zoom_mod, zoom_add, zoom_base, zoom, lastFacing, fsin, fcos, isRotating, isOutdoor, inInstance
local svar

-- icon frames pool
local function getbtn()
 local btn = frames_pool[1]
 if btn then tremove(frames_pool, 1) end
 return btn
end

local function freebtn(btn)
 if not btn then return end

 if not tContains(frames_pool, btn) then
  frames_pool[#frames_pool + 1] = btn
 end
 btn:Hide()
end
--

-- icon events
local function mapbutton_onmouseup(self, mousebutton)
 if not IsControlKeyDown() or mousebutton ~= "RightButton" then
  local p = self:GetParent()
  if self:GetParent() == wmf then
   WorldMapButton_OnClick(WorldMapButton, mousebutton)
  else
   Minimap_OnClick(Minimap, mousebutton)
  end
  return
 end

 local data = self.data

 if data then
  local x = data[5][2] / 100
  local y = data[5][3] / 100
  addonTable.setTomTomWP(data[5][1], nil, x, y, { title = data[2]})
 end
end

local function mapbutton_onenter(self)
 local data = self.data

 if not data then return end

 DailyGlobalCheck_maptooltip:SetOwner(self,"ANCHOR_BOTTOM")
 DailyGlobalCheck_maptooltip:ClearLines()

 DailyGlobalCheck_maptooltip:AddLine("|cff00AAFFDaily Global Check|r")
 DailyGlobalCheck_maptooltip:AddLine(AddColor(self.list and self.list.Title or "","LIGHT_BLUE"))
 DailyGlobalCheck_maptooltip:AddLine(data[3].." "..data[2])
 DailyGlobalCheck_maptooltip:AddLine(data[4])
 local desc = self.info and self.info.desc
 if desc then
  DailyGlobalCheck_maptooltip:AddLine(" ")
  DailyGlobalCheck_maptooltip:AddLine(AddColor(desc, "LIGHT_GREEN"))
 end
 DailyGlobalCheck_maptooltip:Show()

 DailyGlobalCheck_maptooltip.info:SetOwner(DailyGlobalCheck_maptooltip, "ANCHOR_BOTTOM")
 local info = DailyGlobalCheck.QuestFunctions:GetOverlayInfo(data.questID, data)
 if (info and DailyGlobalCheck:OverlayInfoToTooltip(DailyGlobalCheck_maptooltip.info, info)) then
  DailyGlobalCheck_maptooltip.info:Show()
 else
  DailyGlobalCheck_maptooltip.info:Hide()
 end
end

local function mapbutton_onleave()
 DailyGlobalCheck_maptooltip:Hide()
end
--

local counter = 0
local function createButton(t)
 local f = getbtn()
 if not f then
  f = CreateFrame("Frame", "DailyGlobalCheck_MapIcon"..counter)
  f:SetScript("OnMouseUp", mapbutton_onmouseup) 
  f:SetScript("OnEnter", mapbutton_onenter)
  f:SetScript("OnLeave", mapbutton_onleave)
  f.tex = f:CreateTexture()
  f.tex:SetAllPoints()
  counter = counter + 1
 end

 f:SetParent(t == "worldmap" and wmf or mmf)
 local size = svar.UI[t.."_icon_size"]
 f:SetSize(size, size)
 f:EnableMouse(svar.UI[t.."_icon_interact"])
 return f
end

local function removeButton(i, index)
 if not i then return end
 
 if index then
  if i["button"..index] then
   freebtn(i["button"..index])
   i["button"..index] = nil
  end
  return
 end
 
 local k = 1
 while i["button"..k] do
  freebtn(i["button"..k])
  i["button"..k] = nil
  k = k + 1
 end
end

function maps:RemoveList(t, list)
 if not lists:isList(list) then return end

 for _,v in pairs(icons[t]) do
  for j,i in pairs(v) do
   if i.list == list then
    removeButton(i)
    i.visible = false
   end
  end
 end
end

local function AddButton(t, v, list)
 if not icons[t][map_data[v[5][1]].id] then
  icons[t][map_data[v[5][1]].id] = {}
 end
 if not icons[t][map_data[v[5][1]].id][v] then
  icons[t][map_data[v[5][1]].id][v] = { list = list  }
 end
 icons[t][map_data[v[5][1]].id][v].visible = true
end

local function AddList(t, list)

 for _, page in pairs(list.Order) do
  for _, group in pairs(page) do
   for i = 2, #group do
	local v = DailyGlobalCheck:getQData(group[i], list.Title)
	if v and v[5] and map_data[v[5][1]] then
	 if not DailyGlobalCheck.isquestcompleted(group[i], true, list) and addonTable.isshown(group[i], list.Title) then
      AddButton(t, v, list)
	 end
	end
   end
  end
 end
end

function maps:UpdateList(t, list, clear)
 if clear then
  maps:RemoveList(t, list)
 end

 if lists:isActive(list) and lists:getShowMapIcons(list, t) then
  AddList(t, list)
 elseif not clear then
  maps:RemoveList(t, list)
 end

 if t == "worldmap" then
  wmf.update()
 else
  mmf.update(true)
 end
end

function maps:Update(t)
 DailyGlobalCheck:foreachLists(function(k,v)
  maps:UpdateList(t, list)
 end)
end

-- worldmap
local function updateWorldmapIcon(i, v, current_data)
 local inDungeon = dungeon_data.id ~= 0

 local k = 1
 while v[5][k] do
  local index = ceil(k / 4)

  local iconInDungeon = type(v[5][k + 3]) == "table" and v[5][k + 3].level
  local iconInThisDungeon = inDungeon and iconInDungeon and iconInDungeon == dungeon_data.id

  local x, y, wx, wy, rect
 
  if iconInThisDungeon then
   rect = dungeon_data
   wx = current_data.w * v[5][k + 1] / 100 + current_data.l
   wy = current_data.h * v[5][k + 2] / 100 + current_data.t
   x = (wx - rect.l) / (rect.r - rect.l)
   y = -(wy - rect.t) / (rect.b - rect.t)
  else
   local data = map_data[v[5][k]]
   rect = current_data
   wx = data.w * v[5][k + 1] / 100 + data.l
   wy = data.h * v[5][k + 2] / 100 + data.t
   x = (wx - rect.l) / rect.w
   y = -(wy - rect.t) / rect.h
  end
 
  if ((not inDungeon and not iconInDungeon) or iconInThisDungeon) and
      wx < rect.l and wx > rect.r and wy < rect.t and wy > rect.b and i.visible then
   local btn = i["button"..index]
   if not btn then
    btn = createButton("worldmap")
    btn.tex:SetTexture((type(v[5][k + 3]) == "table" and v[5][k + 3].icon) or v[8] or T.dgcicon)
	i["button"..index] = btn
   end
   btn.data = v
   btn.list = i.list
   btn.info = v[5][k + 3]
   btn:SetPoint("CENTER", wmf, "TOPLEFT", wmf.w * x, wmf.h * y)
   btn:Show()
  elseif i["button"..index] then
   removeButton(i, index)
  end
  k = k + 4
 end
end

local function checkQuestCompletion(k, v)
 return k.questID and DailyGlobalCheck.isquestcompleted(k.questID, true, v.list)
end

local last_check = 0
local function updateWorldmap()
 local current_data 
 wmf.w, wmf.h = wmf:GetWidth(), wmf:GetHeight()

 dungeon_data.id, dungeon_data.r,
 dungeon_data.b, dungeon_data.l, dungeon_data.t = GetCurrentMapDungeonLevel()

 current_data = map_data[GetCurrentMapAreaID()]

 -- general check every 2~ minutes to be sure we hide every completed quest icon
 if not InCombatLockdown() then
  local now = time()
  if now > last_check + 0 then
   for t,v in pairs(icons) do
    for j,i in pairs(v) do
	 -- we'll be checking the current instance later in this function
	 if not current_data or j ~= current_data.id then
	  for k,data in pairs(i) do
       if checkQuestCompletion(k, data) then
	    removeButton(data)
	    v[j] = nil
	   end
	  end
	 end
	end
   end
   last_check = now
  end
 end
 
 if icons.worldmap[wmf.last_instanceID] and (not current_data or wmf.last_instanceID ~= current_data.id or dungeon_data.id ~= wmf.last_dungeon) then
  for _,i in pairs(icons.worldmap[wmf.last_instanceID]) do
   removeButton(i)
  end
 end

 if current_data and WorldMapButton:IsVisible() and icons.worldmap[current_data.id] then
  for v, i in pairs(icons.worldmap[current_data.id]) do
   if checkQuestCompletion(v, i) then
    removeButton(i)
    icons.worldmap[current_data.id][v] = nil
	if icons.minimap[current_data.id] and icons.minimap[current_data.id] then
	 removeButton(icons.minimap[current_data.id][v])
	 icons.minimap[current_data.id][v] = nil
	end
   else
    updateWorldmapIcon(i, v, current_data)
   end
  end
 end
 wmf.last_instanceID = current_data and current_data.id
 wmf.last_dungeon = dungeon_data.id
end
--
wmf.update = updateWorldmap

-- minimap
local function updateMinimapIcon(i, v)

 local k = 1
 while v[5][k] do
  local index = ceil(k / 4)
  local data = map_data[v[5][k]]
  
  local wx = data.w * v[5][k + 1] / 100 + data.l
  local wy = data.h * v[5][k + 2] / 100 + data.t
  
  local dx, dy = (player_last_x - wx) * zoom, (player_last_y - wy) * zoom
  
  if isRotating then
   dx, dy = dx * fcos - dy * fsin, dx * fsin + dy * fcos
  end

  local isRound
  if shape then
   local i = dx < 0 and 1 or 3
   if dy < 0 then
    isRound = shape[i]
   else
    isRound = shape[i + 1]
   end
  else
   isRound = true
  end
  
  local dist = isRound and (dx ^ 2 + dy ^ 2) / 0.8 or max(dx ^ 2, dy ^ 2) / 0.8
  
  local x, y = dx * mmWidth / 2, -dy * mmHeight / 2
  if dist <= 1 and i.visible then
   local btn = i["button"..index]
   if not btn then
    btn = createButton("minimap")
	btn.tex:SetTexture((v[5][k + 3] and v[5][k + 3].icon) or v[8] or T.dgcicon)
	i["button"..index] = btn
   end
   btn.data = v
   btn.list = i.list
   btn.info = v[5][k + 3]
   btn:SetPoint("CENTER", mmf, "CENTER", x, y)
   btn:Show()
  elseif i["button"..index] then
   removeButton(i, index)
  end
  k = k + 4
 end
end

local function updateMinimap(forced)

 if not icons.minimap[player_instanceID] then return end

 local py, px = UnitPosition("player")
 
 local z = Minimap:GetZoom()
 if not isOutdoor then
  zoom_mod  = z == 4 and 1.6 or 1.2
  zoom_add  = z == 5 and 1 or 0
 end
 
 zoom = zoom_base / ((5 - z) * zoom_mod + zoom_add)

 local facing = isRotating and GetPlayerFacing() or 0

 if px ~= player_last_x or py ~= player_last_y or zoom ~= lastZoom or (isRotating and facing ~= lastFacing) or forced then

  shape = GetMinimapShape and minimapShapes[GetMinimapShape()]

  player_last_x = px
  player_last_y = py
  lastZoom = zoom
  lastFacing = facing
 
  if isRotating then
   fsin = sin(facing)
   fcos = cos(facing)
  end

  if icons.minimap[player_instanceID] then
   for v, i in pairs(icons.minimap[player_instanceID]) do
    updateMinimapIcon(i, v)
   end
  end
 end
end

mmf.update = updateMinimap

local last_time = 0
function minimapOnUpdate(_, elapsed)
 last_time = last_time + elapsed
 if last_time > 0.06 then
  updateMinimap()
  last_time = 0
 end
end

local function updateMinimapZoom()
 local z = Minimap:GetZoom()
 if GetCVar("minimapZoom") == GetCVar("minimapInsideZoom") then
  Minimap:SetZoom(z + (z < 4 and 1 or -1))
 end
 isOutdoor = tonumber(GetCVar("minimapZoom")) == Minimap:GetZoom()
 if isOutdoor then
  zoom_base = 0.015
  zoom_mod  = 0.5
  zoom_add  = 1
 else
  zoom_base = 0.04
 end
 Minimap:SetZoom(z)
end
--

local function updateIconsSize(t, value)
 local size = value
 for _,v in pairs(icons[t]) do
  for _,i in pairs(v) do
   local k = 1
   while i["button"..k] do
    i["button"..k]:SetSize(size, size)
	k = k + 1
   end
  end
 end
end

local function updateInteractable(t, value)
 for _,v in pairs(icons[t]) do
  for _,i in pairs(v) do
   local k = 1
   while i["button"..k] do
    i["button"..k]:EnableMouse(value)
	k = k + 1
   end
  end
 end
end

local function updateVisibility(t, value)
 local f = t == "worldmap" and wmf or mmf
 if value then
  f:Show()
  if t == "minimap" and not inInstance then
   f:SetScript("OnUpdate", minimapOnUpdate)
  end
  f.update(true)
 else 
  f:Hide()
  if t == "minimap" then
   f:SetScript("OnUpdate", nil)
  end
 end
end

local function updateVisibility_List(t, _, list)
 maps:UpdateList(t, list)
 if t == "worldmap" then
  updateWorldmap()
 else
  updateMinimap(true)
 end
end

function maps:Initialize()
 svar = DailyGlobalCheck_Options

 wmf:SetAllPoints()

 mmWidth  = 0
 mmHeight = 0
 mmf:SetScript("OnSizeChanged", function()
  mmWidth = mmf:GetWidth()
  mmHeight = mmf:GetHeight()
 end)

 mmf:SetAllPoints()
 isRotating = GetCVar("rotateMinimap") == "1"
 mmf:SetScript("OnUpdate", minimapOnUpdate)

 CreateFrame("GameTooltip","DailyGlobalCheck_maptooltip", nil, "GameTooltipTemplate")
 DailyGlobalCheck_maptooltip:SetFrameStrata("TOOLTIP")
 DailyGlobalCheck_maptooltip:SetScale(0.8)
 DailyGlobalCheck_maptooltip.info = CreateFrame("GameTooltip", "DailyGlobalCheck_maptooltipInfo", DailyGlobalCheck_maptooltip, "GameTooltipTemplate")
 DailyGlobalCheck_maptooltip.info:SetScale(0.9)

 DGCEventFrame:RegisterEvent("WORLD_MAP_UPDATE")
 function DGCEventFrame.WORLD_MAP_UPDATE(...)
  updateWorldmap()
 end

 DGCEventFrame:RegisterEvent("MINIMAP_UPDATE_ZOOM")
 function DGCEventFrame.MINIMAP_UPDATE_ZOOM(...)
  updateMinimapZoom()
 end
 
 DGCEventFrame:RegisterEvent("CVAR_UPDATE")
 function DGCEventFrame.CVAR_UPDATE(...)
  if ... == "ROTATE_MINIMAP" then
   isRotating = select(2, ...) == "1"
   updateMinimap()
  end
 end

 local function setPlayerZone()
  local prev_mapID = GetCurrentMapAreaID()
  SetMapToCurrentZone()
  local mapID = GetCurrentMapAreaID()
  player_mapID = mapID
  player_instanceID = map_data[mapID] and map_data[mapID].id
  SetMapByID(prev_mapID)
 end

 DGCEventFrame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
 DGCEventFrame:RegisterEvent("ZONE_CHANGED")
 DGCEventFrame:RegisterEvent("ZONE_CHANGED_INDOORS")
 DGCEventFrame.ZONE_CHANGED          = setPlayerZone
 DGCEventFrame.ZONE_CHANGED_INDOORS  = setPlayerZone
 DGCEventFrame.ZONE_CHANGED_NEW_AREA = setPlayerZone
 
 -- minimize CPU usage inside instances
 local function toggleMinimap()
  local now = IsInInstance()
  if now and not inInstance then
   mmf:SetScript("OnUpdate", nil)
  elseif inInstance and not now then
   mmf:SetScript("OnUpdate", minimapOnUpdate)
  end
  inInstance = now
 end
 DGCEventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
 DGCEventFrame.PLAYER_ENTERING_WORLD = toggleMinimap
 --
 
 updates._icon_size     = updateIconsSize
 updates._icon_interact = updateInteractable
 updates.show_icons     = updateVisibility
 updates.show_mapicons_ = updateVisibility_List -- plugin option
 
 local function option_changed(opt, value, list)
  -- not interested in plugin generic options
  if type(opt) ~= "string" then return end
 
  local t, func

  local i, j = opt:find("worldmap")
  if not i then
   t = "minimap"
   i, j = opt:find(t)
   func = opt:gsub(t, "")
  else
   t = "worldmap"
   func = opt:gsub(t, "")
  end

  if updates[func] then
   updates[func](t, value, list)
  end
 end
 
 DGCEventFrame:Hook("OPTION_CHANGED", option_changed)
 DGCEventFrame:Hook("OPTION_CHANGED_PLUGIN", option_changed)

 local function updateicons(list)
  maps:UpdateList("worldmap", list)
  maps:UpdateList("minimap", list)
 end
 
 DGCEventFrame:Hook("PLUGIN_LOADED", updateicons)
 
 option_changed("show_worldmapicons", svar.show_worldmapicons)
 option_changed("show_minimapicons", svar.show_minimapicons)
 
 updateMinimapZoom()
 setPlayerZone()
end