-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

local tinsert = table.insert

local addonName, addonTable = ...

local L = addonTable.L
local C = addonTable.C
local ng = NyxGUI(C.NyxGUIVersion)
local AddColor = addonTable.AddColor

local constructors = DailyGlobalCheck.Constructors
local modes        = DailyGlobalCheck.ModeFunctions
local options      = DailyGlobalCheck.OptFunctions
local tools        = DailyGlobalCheck.Tools
local help         = DailyGlobalCheck.HelpFunctions

local mf, setup_functions

-- quest buttons
local function setup_panel_1(f)
 f.buttons = {}
 
 f.HideQuestButtons = function(self, from)
  local length = #(self.buttons)
  
  if length == 0 or (from and from > length) then return end

  if not from or from == 0 then 
   from = 1
  end

  for i = from, length do
   self.buttons[i]:Hide()
  end
 end
end

local function createPanel(i)
 local f = CreateFrame("Frame", "DGCMainframePanel"..i, mf)
 
 f:SetPoint("TOPLEFT", mf, "TOPLEFT", 1, -70)
 f:SetPoint("BOTTOMRIGHT", mf, "BOTTOMRIGHT", -1, 70)
 
 if setup_functions[i] then
  setup_functions[i](f)
 end
 
 mf.panels[i] = f
end

function modes:Hide(mode)
 if mode and mf.panels[C.modes[mode].panel] then
  mf.panels[C.modes[mode].panel]:Hide()
 else
  for _,v in pairs(mf.panels) do
   v:Hide()
  end
 end
end

function modes:Set(mode)
 if not mf then return end

 if not mf.panels then mf.panels = {} end

 help:Hide()
 modes:Hide()
 DailyGlobalCheck.mode = mode
 
 -- create a panel only if needed
 if not mf.panels[C.modes[mode].panel] then
  createPanel(C.modes[mode].panel)
 end
 
 local panel = mf.panels[C.modes[mode].panel]
 panel:Show()
 
 DailyGlobalCheck:Refresh()
end

local function modebutton_onclick(self)
 modes:Set(self.tag)
end

function constructors:ModesFrame(p)

 mf = DGC_Mainframe
 setup_functions = { setup_panel_1,
                     options.setup_quests_panel,
					 options.setup_plugin_panel,
					 options.setup_map_panel,
					 tools.setup_list_visibility_panel,
                     tools.setup_edit_mode_panel,
                     tools.setup_edit_quest_panel }

 local f = CreateFrame("Frame", "DGCModesFrame", p)
 f:SetFrameStrata("HIGH")
 f:SetBackdrop({ bgFile = "Interface\\Buttons\\GreyscaleRamp64", 
                          edgeFile = "Interface\\Buttons\\WHITE8X8", 
					   edgeSize = 1
					   })
 f:SetBackdropColor(0.07,0.07,0.07,0.9)
 f:SetBackdropBorderColor(0.1,0.1,0.1,0.9)
 f:SetPoint("LEFT", p, "RIGHT")
 f:SetHeight(390)
 f:SetWidth(200)

 local last_t
 local offsetY, h = -25, 25
 for k,v in pairs(C.modes) do
  if not v.hidden then
   if v.t ~= nil and last_t ~= v.t then
    local lbl = ng:New(addonName, "Label", "DGCModesFrameLabel"..v.t, f)
    lbl:SetPoint("TOPLEFT", f, "TOPLEFT", 0, -offsetY - h)
    lbl:SetPoint("TOPRIGHT", f, "TOPRIGHT", 0, -offsetY - h)
    lbl:SetJustifyH("CENTER")
    lbl:SetHeight(h)
    lbl:SetText(v.t)
    offsetY = offsetY + h * 2
    last_t = v.t
   else
    offsetY = offsetY + h
   end
   
   local btn = ng:New(addonName, "Button", "DGCModesFrameTabbutton"..k, f)
   btn:SetPoint("TOPLEFT", f, "TOPLEFT", 20, -offsetY)
   btn:SetPoint("TOPRIGHT", f, "TOPRIGHT", -20, -offsetY)
   btn:SetText(v.name)
   btn.tag = k
   btn:SetScript("OnClick", modebutton_onclick)
   
   --btn:SetBackdropColor(0.18, 0.18, 0.18, 0.9)
   --btn:SetBackdropBorderColor(0, 0, 0, 0)
  end
 end
 
 f:SetHeight(offsetY + h + 5)
 f:Hide()
  
 return f
end